/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SystemUtil.java 446 2006-05-19 14:13:50Z ddesjardins $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.kernel.api.server.util;

import java.io.File;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;

import org.ow2.petals.kernel.api.server.PetalsException;

/**
 * This utility class provides facilities to get system information.
 * 
 * @version $Rev: 446 $ $Date: 2006-05-19 16:13:50 +0200 (ven, 19 mai 2006) $
 * @since Petals 1.0
 * @author alouis, Rafael Marins
 * @author ddesjardins - eBMWebsourcing
 * @author Marc Dutoo - Open Wide
 * @author chamerling - eBMWebsourcing
 * @author roland NAUDIN - eBMWebsourcing
 */
public final class SystemUtil {

    /**
     * Environment variable name to store the PEtALS home directory.
     */
    public static final String ENVIRONMENT_INSTALL_PATH = "PETALS_HOME";

    /**
     * Java property name to store the PEtALS home directory.
     */
    public static final String PROPERTY_INSTALL_PATH = "petals.home";

    /**
     * The PEtALS home directory. <code>null</code> if not found.
     */
    private static File petalsHome;

    static {
        // first try to resolve install directory set in the environment
        SystemUtil.petalsHome = SystemUtil.resolveInstallDirectoryFromEnvironment();
        // otherwise, guess install directory using the framework
        if (SystemUtil.petalsHome == null) {
            try {
                SystemUtil.petalsHome = SystemUtil.resolveInstallDirectoryFromFramework();
            } catch (final PetalsException e) {
                // NOP
            }
        }
    }

    /**
     * No constructor because <code>SystemUtil</code> is an utility class.
     * 
     */
    private SystemUtil() {
        super();

    }

    /**
     * Get the PEtALS home directory.
     * 
     * @return The PEtALS home directory
     */
    public static File getPetalsInstallDirectory() {

        if (SystemUtil.petalsHome == null) {
            throw new RuntimeException("Petals installation directory can not be found. "
                    + "Set a PETALS_HOME variable or use petals.home JAVA property");
        }

        return SystemUtil.petalsHome;
    }

    /**
     * Try to get the petals'home environment variable : petals.home system
     * property, or PETALS_HOME env variable
     * 
     * @return the related directory, or null
     */
    private static File resolveInstallDirectoryFromEnvironment() {
        File directory = null;
        String petalsHomeValue = System.getProperty(SystemUtil.PROPERTY_INSTALL_PATH);
        if (petalsHomeValue == null || (petalsHomeValue.trim().length() == 0) ) {
            petalsHomeValue = System.getenv(SystemUtil.ENVIRONMENT_INSTALL_PATH);
            // not null or empty
            if (petalsHomeValue != null && (petalsHomeValue.trim().length() > 0) ) {
                final File theHome = new File(petalsHomeValue);
                // exists and it's a directory
                if (theHome.exists() || theHome.isDirectory()) {
                    directory = theHome;
                }
            }
        }
        return directory;
    }

    /**
     * Try to find the current install directory
     * 
     * @return the directory or null if a problem occured
     * @throws PetalsException
     *             should not be thrown
     */
    private static File resolveInstallDirectoryFromFramework() throws PetalsException {
        // guess from the location of the current JAR
        File directory = null;
        final URL url = SystemUtil.class.getProtectionDomain().getCodeSource().getLocation();
        if (url != null) {
            try {
                final URI baseURI = new URI(url.toString()).resolve("..");
                directory = new File(baseURI);
            } catch (final URISyntaxException e) {
                throw new PetalsException(e);
            }
        }
        return directory;
    }

}
